# This file has been generated


build_system = 'AppVeyor'
build_id = '23435337'
build_number = '288'
build_version = '1.0.288'
job_id = 'ruoxypr180qdq05m'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/ruoxypr180qdq05m'
